import 'package:flutter/foundation.dart';

import '../../../core/core.dart';
import '../../../m4e_forms/m4e_forms.dart';
import '../../../models/models.dart';
import '../../api/apis.dart';

class TransactionServiceUtil {
  final WalletApi _walletApi;

  TransactionServiceUtil({
    @required WalletApi walletApi,
  })  : assert(walletApi != null),
        _walletApi = walletApi;

  /// query and run query for reciepient wallet-wrapper on their phone number
  ///
  /// creates a new wallet-wrapper with receipient's phone number if no
  /// wallet associated with recipient phone number is returned and
  /// initiate transaction with the newly created wallet
  ///
  /// in cases where a wallet-wrapper already exists for receipient's phone
  /// number, proceed with initiating a transaction with returned wallet
  Future<M4eTransactionForm> mobileMoneyProviderTransactionMapper(
      M4eUniqueId userId, M4eTransactionForm transaction) async {
    try {
      M4eTransactionForm _modifiedTransaction;

      final _walletDetail = transaction.to.detail as MobileMoneyWalletDetail;

      final _composedWalletSearchForm = MobileMoneyProviderSearchForm(
        provider: _walletDetail.provider,
        phoneNumber: _walletDetail.phoneNumber,
      );

      final _runQueryUniqueId =
          await _walletApi.queryUserWallets(_composedWalletSearchForm);

      // returns list of wallets gotten from running search-returned id
      final _walletList = await _walletApi.runSearchQuery(_runQueryUniqueId);

      if (_walletList.wallets.isEmpty) {
        /// create wallet
        final _walletUniqueId = await _walletApi.createWallet(
            userId, transaction.to.toWalletFormType());

        /// update transaction `to` wallet-id to the newly created one
        _modifiedTransaction = transaction.copyWith(
            to: transaction.to.copyWith(id: _walletUniqueId));

        return _modifiedTransaction;
      } else {
        /// existing wallet was returned
        /// map the first wallet-object returned to the transaction-to wallet
        _modifiedTransaction =
            transaction.copyWith(to: _walletList.wallets.first);

        return _modifiedTransaction;
      }
    } catch (_) {
      rethrow;
    }
  }
}
