import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../models/models.dart';

class M4eWalletForm extends Equatable {
  final M4eUniqueId id;
  final String label;
  final M4eCurrencyType currency;
  final M4eWalletProviderType type;
  final M4eWalletDetailType detail;

  const M4eWalletForm._(
      this.id, this.label, this.currency, this.type, this.detail);

  factory M4eWalletForm({
    M4eUniqueId id,
    @required String label,
    @required M4eCurrencyType currency,
    @required M4eWalletProviderType type,
    @required M4eWalletDetailType detail,
  }) {
    assert(label != null, 'M4eWalletForm [label] should not be null');
    assert(currency != null, 'M4eWalletForm [currency] should not be null');
    assert(type != null, 'M4eWalletForm [type] should not be null');
    assert(detail != null, 'M4eWalletForm [detail] should not be null');

    // TODO: consider validate [M4eWalletProviderType] to its detail-id

    // validate [M4eWalletProviderType] to [M4eWalletDetailType] relationship
    if (type == M4eWalletProviderType.M4E &&
        detail is MobileMoneyWalletDetail) {
      // TODO: update [M4eWalletProviderType] to M4E from BASCULE
      throw AssertionError(
          'M4eWalletForm [detail] should not be of type [OtherWalletDetail] when [M4eWalletProviderType] is BASCULE! \nUse [M4eWalletDetail] as [M4eWalletDetailType] type instead');
    }

    return M4eWalletForm._(id ?? M4eUniqueId(), label, currency, type, detail);
  }

  factory M4eWalletForm.fromJson(Map<String, dynamic> json) {
    return M4eWalletForm._(
      M4eUniqueId.fromUniqueString(json['id']),
      json['wallet']['label'],
      M4eCurrencyType.parse(json['wallet']['currency']),
      M4eWalletProviderType.parse(json['wallet']['type']),
      M4eWalletDetailType.fromJson(json['detail']),
    );
  }

  Map<String, dynamic> toJson() {
    return {
      'label': this.label,
      'currency': this.currency.toString(),
      'type': this.type.toString(),
      'detail': this.detail.map(
            m4eWalletDetail: (detail) => detail.toJson(),
            mobileMoneyWalletDetail: (detail) => detail.toJson(),
          ),
    };
  }

  @override
  List<Object> get props => [id, label, currency, type];

  @override
  bool get stringify => true;
}
