import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';

import '../core/core.dart';
import 'models.dart';

class M4eTransaction extends Equatable {
  final M4eUniqueId id;
  final M4eWallet from;
  final M4eWallet to;
  final M4eAmount amount;
  final DateTime at;
  final M4eTransactionType type;
  final Map<String, dynamic> meta;

  const M4eTransaction({
    @required this.id,
    @required this.from,
    @required this.to,
    @required this.amount,
    @required this.at,
    @required this.type,
    @required this.meta,
  });

  factory M4eTransaction.fromJson(Map<String, dynamic> json) {
    final _transaction = json['transaction'];

    return M4eTransaction(
      id: M4eUniqueId.fromUniqueString(json['id']),
      from: M4eWallet.fromJson(_transaction['flow']['from']),
      to: M4eWallet.fromJson(_transaction['flow']['to']),
      amount: M4eAmount.fromJson(_transaction['flow']['amount']),
      at: DateTimeConverter.fromTimestampSinceEpoch(_transaction['flow']['at']),
      type: M4eTransactionType.parse(_transaction['txType']),
      meta: _transaction['meta'],
    );
  }

  Map<String, dynamic> toJson() {
    final flow = {
      'from': this.from.toJson(),
      'to': this.to.toJson(),
      'amount': this.amount.toJson(),
      'at': this.at.toTimestampSinceEpoch(),
    };

    return {
      'id': this.id.toString(),
      'transaction': {
        'flow': flow,
        'txType': this.type,
        'meta': this.meta,
      },
    };
  }

  @override
  List<Object> get props => [
        id,
        from,
        to,
        amount,
        at,
        type,
        meta,
      ];

  @override
  String toString() {
    return 'M4eTransaction(id: $id, from: $from, to: $to, amount: $amount, at: $at, type: $type, meta: $meta)';
  }
}

class M4eTransactionList extends Equatable {
  final List<M4eTransaction> transactions;

  const M4eTransactionList({@required this.transactions});

  factory M4eTransactionList.fromJson(List<dynamic> json) {
    // TODO: add `data` object in json structure
    // TODO: change `List<dynamic>` to Map<String, dynamic>

    return M4eTransactionList(
      transactions:
          json?.map((e) => M4eTransaction.fromJson(e))?.toList() ?? [],
    );
  }

  // TODO: change `List<dynamic>` to Map<String, dynamic>
  List<dynamic> toJson() {
    return this.transactions?.map((e) => e.toJson())?.toList() ?? [];
  }

  @override
  List<Object> get props => [transactions];

  @override
  String toString() => 'M4eTransactionList(transactions: $transactions)';
}
