import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/m4e_forms/m4e_forms.dart';
import 'package:m4e_sdk/src/models/models.dart';

class WalletTestModel {
  static final uniqueId =
      M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');
  static final otherUniqueId =
      M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e1');

  static final authToken = M4eAuthToken(id: uniqueId, token: 'test token');

  static final phoneNumber =
      M4ePhoneNumber(country: 'GHANA', number: '+233708788110');

  static final m4eUser = M4eUser(
    id: uniqueId,
    shortName: 'test',
    firstName: 'test',
    lastName: 'test',
    phoneNumber: phoneNumber,
    joined: DateTime.now(),
  );

  static final m4eOtherUser = M4eUser(
    id: otherUniqueId,
    shortName: 'test',
    firstName: 'test',
    lastName: 'test',
    phoneNumber: phoneNumber,
    joined: DateTime.now(),
  );

  static final tWallets = <M4eWallet>[
    M4eWallet(
      id: M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0'),
      walletInfo: M4eWalletInfo(
        label: 'W1',
        currency: M4eCurrencyType.GHS,
        type: M4eWalletProviderType.M4E,
      ),
      detail: M4eWalletDetail(
        id: M4eUniqueId.fromUniqueString(
            '46e87882-4ff8-4770-9846-870fbbcc35e0'),
      ),
    ),
    M4eWallet(
      id: M4eUniqueId.fromUniqueString('9e8b5ecc-9f7e-4e06-ac3d-05b11a91da88'),
      walletInfo: M4eWalletInfo(
        label: 'W2',
        currency: M4eCurrencyType.GHS,
        type: M4eWalletProviderType.MTN,
      ),
      detail: MobileMoneyWalletDetail(
          provider: M4eWalletProviderType.MTN, phoneNumber: phoneNumber),
    ),
  ];

  static final _tSearchAnonymousWalletsByPhoneNumberResult = <M4eWallet>[
    M4eWallet(
      id: M4eUniqueId.fromUniqueString('9e8b5ecc-9f7e-4e06-ac3d-05b11a91da88'),
      walletInfo: M4eWalletInfo(
        label: 'W2',
        currency: M4eCurrencyType.GHS,
        type: M4eWalletProviderType.MTN,
      ),
      detail: MobileMoneyWalletDetail(
          provider: M4eWalletProviderType.MTN, phoneNumber: phoneNumber),
    ),
  ];

  static final tWalletBalance =
      M4eBalance(value: 1.0, unit: M4eCurrencyType.GHS);

  static final tWalletListEmpty = M4eWalletList(wallets: []);
  static final tWalletList = M4eWalletList(wallets: tWallets);
  static final tWalletListAnonymousSearchByPhoneNumber =
      M4eWalletList(wallets: _tSearchAnonymousWalletsByPhoneNumberResult);

  static final tM4eWalletForm = M4eWalletForm(
    label: 'test',
    currency: M4eCurrencyType.GHS,
    type: M4eWalletProviderType.M4E,
    detail: M4eWalletDetail(
      id: M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0'),
    ),
  );
  static final tMTNWalletForm = M4eWalletForm(
    label: 'test',
    currency: M4eCurrencyType.GHS,
    type: M4eWalletProviderType.MTN,
    detail: MobileMoneyWalletDetail(
        provider: M4eWalletProviderType.MTN, phoneNumber: phoneNumber),
  );

  static final ownerWalletAccessPermissionsSet = <M4eAccessPermissionSet>[
    EntityWalletAccess.view,
    EntityWalletAccess.grant,
    EntityWalletAccess.alter,
  ];

  static final otherUsersWalletAccessPermissionsSet = <M4eAccessPermissionSet>[
    EntityWalletAccess.see,
    EntityWalletAccess.pay,
  ];

  // Single search clause ======================================================================================
  static final singleSearchFormTypeCollection = SingleClauseSearchForm(
    clause: SearchClause(
      subjectId: uniqueId,
      type: M4eAccessPermissionType.ENTITY_WALLET,
      encoding: ownerWalletAccessPermissionsSet,
    ),
  );

  // Multiple search clauses ====================================================================================
  static final ownerWalletsAccessSearchClause = SearchClause(
    subjectId: otherUniqueId,
    type: M4eAccessPermissionType.ENTITY_WALLET,
    encoding: ownerWalletAccessPermissionsSet,
  );

  // accessing wallets current user has [SEE, PAY] ENTITYWALLET permissions on
  static final currentUsersWalletAccessSearchClause = SearchClause(
    subjectId: uniqueId,
    type: M4eAccessPermissionType.ENTITY_WALLET,
    encoding: otherUsersWalletAccessPermissionsSet,
  );

  static final multipleSearchFormTypeCollection = MultipleClauseSearchForm(
    conjunction: M4eSearchConjunctionType.AND,
    clauses: <SearchClause>[
      ownerWalletsAccessSearchClause,
      currentUsersWalletAccessSearchClause,
    ],
  );
}
